"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeStack = void 0;
const shared_1 = require("../shared");
class NodeStack {
    constructor(stack = []) {
        this.stack = [...stack];
    }
    push(node) {
        this.stack.push(node);
    }
    pop() {
        return this.stack.pop();
    }
    peek() {
        return this.isEmpty() ? undefined : this.stack[this.stack.length - 1];
    }
    all() {
        return [...this.stack];
    }
    isEmpty() {
        return this.stack.length === 0;
    }
    clone() {
        return new NodeStack(this.stack);
    }
    toString() {
        return this.toStringArray().join(' > ');
    }
    toStringArray() {
        return this.stack.map((node) => {
            switch (node.kind) {
                case 'rootNode':
                    return 'Root';
                case 'programNode':
                    return node.name ? `Program: ${node.name}` : 'Unnamed Program';
                case 'accountNode':
                    return node.name ? `Account: ${node.name}` : 'Unnamed Account';
                case 'instructionNode':
                    return node.name
                        ? `Instruction: ${node.name}`
                        : 'Unnamed Instruction';
                case 'instructionAccountNode':
                    return node.name
                        ? `Instruction Account: ${node.name}`
                        : 'Unnamed Instruction Account';
                case 'definedTypeNode':
                    return node.name
                        ? `Defined Type: ${node.name}`
                        : 'Unnamed Defined Type';
                case 'errorNode':
                    return node.name ? `Error: ${node.name}` : 'Unnamed Error';
                case 'enumEmptyVariantTypeNode':
                case 'enumStructVariantTypeNode':
                case 'enumTupleVariantTypeNode':
                    return node.name ? `Variant: ${node.name}` : 'Variant';
                case 'numberWrapperTypeNode':
                    return `Number Wrapper: ${node.wrapper.kind}`;
                case 'structFieldTypeNode':
                    return node.name ? `Field: ${node.name}` : 'Field';
                default:
                    return (0, shared_1.titleCase)(node.kind.replace(/(Type)?Node$/, ''));
            }
        });
    }
    matchesWithNames(names) {
        const remainingNames = [...names].map(shared_1.mainCase);
        this.stack.forEach((node) => {
            const nodeName = node.name;
            if (nodeName &&
                remainingNames.length > 0 &&
                remainingNames[0] === (0, shared_1.mainCase)(nodeName)) {
                remainingNames.shift();
            }
        });
        return remainingNames.length === 0;
    }
}
exports.NodeStack = NodeStack;
//# sourceMappingURL=NodeStack.js.map