"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetDefaultValidatorBagVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const NodeStack_1 = require("../NodeStack");
const ValidatorBag_1 = require("../ValidatorBag");
const Visitor_1 = require("../Visitor");
const GetResolvedInstructionInputsVisitor_1 = require("./GetResolvedInstructionInputsVisitor");
class GetDefaultValidatorBagVisitor {
    constructor() {
        this.stack = new NodeStack_1.NodeStack();
        this.program = null;
        this.definedTypes = new Set();
    }
    visitRoot(root) {
        // Register defined types to make sure links are valid.
        nodes
            .getAllDefinedTypes(root)
            .forEach((type) => this.definedTypes.add(type.name));
        this.pushNode(root);
        const bags = root.programs.map((program) => (0, Visitor_1.visit)(program, this));
        this.popNode();
        return new ValidatorBag_1.ValidatorBag().mergeWith(bags);
    }
    visitProgram(program) {
        this.pushNode(program);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!program.name) {
            bag.error('Program has no name.', program, this.stack);
        }
        if (!program.publicKey) {
            bag.error('Program has no public key.', program, this.stack);
        }
        if (!program.version) {
            bag.warn('Program has no version.', program, this.stack);
        }
        if (!program.origin) {
            bag.info('Program has no origin.', program, this.stack);
        }
        bag.mergeWith([
            ...program.accounts.map((node) => (0, Visitor_1.visit)(node, this)),
            ...program.instructions.map((node) => (0, Visitor_1.visit)(node, this)),
            ...program.definedTypes.map((node) => (0, Visitor_1.visit)(node, this)),
            ...program.errors.map((node) => (0, Visitor_1.visit)(node, this)),
        ]);
        this.popNode();
        return bag;
    }
    visitAccount(account) {
        this.pushNode(account);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!account.name) {
            bag.error('Account has no name.', account, this.stack);
        }
        bag.mergeWith([(0, Visitor_1.visit)(account.data, this)]);
        this.popNode();
        return bag;
    }
    visitAccountData(accountData) {
        this.pushNode(accountData);
        const bag = new ValidatorBag_1.ValidatorBag();
        bag.mergeWith([(0, Visitor_1.visit)(accountData.struct, this)]);
        if (accountData.link)
            bag.mergeWith([(0, Visitor_1.visit)(accountData.link, this)]);
        this.popNode();
        return bag;
    }
    visitInstruction(instruction) {
        this.pushNode(instruction);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!instruction.name) {
            bag.error('Instruction has no name.', instruction, this.stack);
        }
        // Check for duplicate account names.
        const accountNameHistogram = new Map();
        instruction.accounts.forEach((account) => {
            var _a;
            if (!account.name) {
                bag.error('Instruction account has no name.', instruction, this.stack);
                return;
            }
            const count = ((_a = accountNameHistogram.get(account.name)) !== null && _a !== void 0 ? _a : 0) + 1;
            accountNameHistogram.set(account.name, count);
            // Only throw an error once per duplicated names.
            if (count === 2) {
                bag.error(`Account name "${account.name}" is not unique in instruction "${instruction.name}".`, instruction, this.stack);
            }
        });
        // Check for cyclic dependencies in account defaults.
        const cyclicCheckVisitor = new GetResolvedInstructionInputsVisitor_1.GetResolvedInstructionInputsVisitor();
        try {
            (0, Visitor_1.visit)(instruction, cyclicCheckVisitor);
        }
        catch (error) {
            bag.error(cyclicCheckVisitor.getError(), instruction, this.stack);
        }
        // Check args.
        bag.mergeWith([(0, Visitor_1.visit)(instruction.dataArgs, this)]);
        // Check extra args.
        bag.mergeWith([(0, Visitor_1.visit)(instruction.extraArgs, this)]);
        const names = [
            ...instruction.dataArgs.struct.fields.map(({ name }) => (0, shared_1.mainCase)(name)),
            ...instruction.extraArgs.struct.fields.map(({ name }) => (0, shared_1.mainCase)(name)),
        ];
        const duplicates = names.filter((e, i, a) => a.indexOf(e) !== i);
        const uniqueDuplicates = [...new Set(duplicates)];
        const hasConflictingNames = uniqueDuplicates.length > 0;
        if (hasConflictingNames) {
            bag.error(`The names of the following instruction arguments are conflicting: ` +
                `[${uniqueDuplicates.join(', ')}].`, instruction, this.stack);
        }
        // Check arg defaults.
        Object.entries(instruction.argDefaults).forEach(([name, defaultsTo]) => {
            if (defaultsTo.kind === 'accountBump') {
                const defaultAccount = instruction.accounts.find((account) => account.name === defaultsTo.name);
                if (defaultAccount && defaultAccount.isSigner !== false) {
                    bag.error(`Argument ${name} cannot default to the bump attribute of ` +
                        `the [${defaultsTo.name}] account as it may be a Signer.`, instruction, this.stack);
                }
            }
        });
        // Check sub-instructions.
        bag.mergeWith(instruction.subInstructions.map((ix) => (0, Visitor_1.visit)(ix, this)));
        this.popNode();
        return bag;
    }
    visitInstructionAccount(instructionAccount) {
        this.pushNode(instructionAccount);
        const bag = new ValidatorBag_1.ValidatorBag();
        this.popNode();
        return bag;
    }
    visitInstructionDataArgs(instructionDataArgs) {
        this.pushNode(instructionDataArgs);
        const bag = new ValidatorBag_1.ValidatorBag();
        bag.mergeWith([(0, Visitor_1.visit)(instructionDataArgs.struct, this)]);
        if (instructionDataArgs.link)
            bag.mergeWith([(0, Visitor_1.visit)(instructionDataArgs.link, this)]);
        this.popNode();
        return bag;
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        this.pushNode(instructionExtraArgs);
        const bag = new ValidatorBag_1.ValidatorBag();
        bag.mergeWith([(0, Visitor_1.visit)(instructionExtraArgs.struct, this)]);
        if (instructionExtraArgs.link)
            bag.mergeWith([(0, Visitor_1.visit)(instructionExtraArgs.link, this)]);
        this.popNode();
        return bag;
    }
    visitDefinedType(definedType) {
        this.pushNode(definedType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!definedType.name) {
            bag.error('Defined type has no name.', definedType, this.stack);
        }
        bag.mergeWith([(0, Visitor_1.visit)(definedType.data, this)]);
        this.popNode();
        return bag;
    }
    visitError(error) {
        this.pushNode(error);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!error.name) {
            bag.error('Error has no name.', error, this.stack);
        }
        if (typeof error.code !== 'number') {
            bag.error('Error has no code.', error, this.stack);
        }
        if (!error.message) {
            bag.warn('Error has no message.', error, this.stack);
        }
        this.popNode();
        return bag;
    }
    visitArrayType(arrayType) {
        this.pushNode(arrayType);
        const bag = (0, Visitor_1.visit)(arrayType.child, this);
        this.popNode();
        return bag;
    }
    visitLinkType(linkType) {
        this.pushNode(linkType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!linkType.name) {
            bag.error('Pointing to a defined type with no name.', linkType, this.stack);
        }
        else if (linkType.importFrom === 'generated' &&
            !this.definedTypes.has(linkType.name)) {
            bag.error(`Pointing to a missing defined type named "${linkType.name}"`, linkType, this.stack);
        }
        this.popNode();
        return bag;
    }
    visitEnumType(enumType) {
        this.pushNode(enumType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (enumType.variants.length === 0) {
            bag.warn('Enum has no variants.', enumType, this.stack);
        }
        enumType.variants.forEach((variant) => {
            if (!variant.name) {
                bag.error('Enum variant has no name.', enumType, this.stack);
            }
        });
        bag.mergeWith(enumType.variants.map((variant) => (0, Visitor_1.visit)(variant, this)));
        this.popNode();
        return bag;
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        this.pushNode(enumEmptyVariantType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!enumEmptyVariantType.name) {
            bag.error('Enum variant has no name.', enumEmptyVariantType, this.stack);
        }
        this.popNode();
        return bag;
    }
    visitEnumStructVariantType(enumStructVariantType) {
        this.pushNode(enumStructVariantType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!enumStructVariantType.name) {
            bag.error('Enum variant has no name.', enumStructVariantType, this.stack);
        }
        bag.mergeWith([(0, Visitor_1.visit)(enumStructVariantType.struct, this)]);
        this.popNode();
        return bag;
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        this.pushNode(enumTupleVariantType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!enumTupleVariantType.name) {
            bag.error('Enum variant has no name.', enumTupleVariantType, this.stack);
        }
        bag.mergeWith([(0, Visitor_1.visit)(enumTupleVariantType.tuple, this)]);
        this.popNode();
        return bag;
    }
    visitMapType(mapType) {
        this.pushNode(mapType);
        const bag = new ValidatorBag_1.ValidatorBag();
        bag.mergeWith([(0, Visitor_1.visit)(mapType.key, this), (0, Visitor_1.visit)(mapType.value, this)]);
        this.popNode();
        return bag;
    }
    visitOptionType(optionType) {
        this.pushNode(optionType);
        const bag = (0, Visitor_1.visit)(optionType.child, this);
        this.popNode();
        return bag;
    }
    visitSetType(setType) {
        this.pushNode(setType);
        const bag = (0, Visitor_1.visit)(setType.child, this);
        this.popNode();
        return bag;
    }
    visitStructType(structType) {
        this.pushNode(structType);
        const bag = new ValidatorBag_1.ValidatorBag();
        // Check for duplicate field names.
        const fieldNameHistogram = new Map();
        structType.fields.forEach((field) => {
            var _a;
            if (!field.name)
                return; // Handled by TypeStructField
            const count = ((_a = fieldNameHistogram.get(field.name)) !== null && _a !== void 0 ? _a : 0) + 1;
            fieldNameHistogram.set(field.name, count);
            // Only throw an error once per duplicated names.
            if (count === 2) {
                bag.error(`Struct field name "${field.name}" is not unique.`, field, this.stack);
            }
        });
        bag.mergeWith(structType.fields.map((field) => (0, Visitor_1.visit)(field, this)));
        this.popNode();
        return bag;
    }
    visitStructFieldType(structFieldType) {
        this.pushNode(structFieldType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (!structFieldType.name) {
            bag.error('Struct field has no name.', structFieldType, this.stack);
        }
        bag.mergeWith([(0, Visitor_1.visit)(structFieldType.child, this)]);
        this.popNode();
        return bag;
    }
    visitTupleType(tupleType) {
        this.pushNode(tupleType);
        const bag = new ValidatorBag_1.ValidatorBag();
        if (tupleType.children.length === 0) {
            bag.warn('Tuple has no items.', tupleType, this.stack);
        }
        bag.mergeWith(tupleType.children.map((node) => (0, Visitor_1.visit)(node, this)));
        this.popNode();
        return bag;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitBoolType(boolType) {
        return new ValidatorBag_1.ValidatorBag();
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitBytesType(bytesType) {
        return new ValidatorBag_1.ValidatorBag();
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitNumberType(numberType) {
        return new ValidatorBag_1.ValidatorBag();
    }
    visitNumberWrapperType(numberWrapperType) {
        this.pushNode(numberWrapperType);
        const bag = (0, Visitor_1.visit)(numberWrapperType.number, this);
        this.popNode();
        return bag;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitPublicKeyType(publicKeyType) {
        return new ValidatorBag_1.ValidatorBag();
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitStringType(stringType) {
        return new ValidatorBag_1.ValidatorBag();
    }
    pushNode(node) {
        this.stack.push(node);
        if (nodes.isProgramNode(node)) {
            this.program = node;
        }
    }
    popNode() {
        const node = this.stack.pop();
        if (node && nodes.isProgramNode(node)) {
            this.program = null;
        }
        return node;
    }
}
exports.GetDefaultValidatorBagVisitor = GetDefaultValidatorBagVisitor;
//# sourceMappingURL=GetDefaultValidatorBagVisitor.js.map