"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetNodeInlineStringVisitor = void 0;
const shared_1 = require("../../shared");
const nodes = __importStar(require("../../nodes"));
const Visitor_1 = require("../Visitor");
const ROOT_PREFIX = 'R';
const PROGRAM_PREFIX = 'P';
const ACCOUNT_PREFIX = 'A';
const INSTRUCTION_PREFIX = 'I';
const TYPE_PREFIX = 'T';
const ERROR_PREFIX = 'E';
class GetNodeInlineStringVisitor {
    visitRoot(root) {
        const children = root.programs.map((program) => (0, Visitor_1.visit)(program, this));
        return `${ROOT_PREFIX}(${children.join(',')})`;
    }
    visitProgram(program) {
        const children = [
            ...program.accounts.map((account) => (0, Visitor_1.visit)(account, this)),
            ...nodes.getAllInstructionsWithSubs(program).map((ix) => (0, Visitor_1.visit)(ix, this)),
            ...program.definedTypes.map((type) => (0, Visitor_1.visit)(type, this)),
            ...program.errors.map((type) => (0, Visitor_1.visit)(type, this)),
        ];
        return `${PROGRAM_PREFIX}[${program.name}](${children.join(',')})`;
    }
    visitAccount(account) {
        const data = (0, Visitor_1.visit)(account.data, this);
        return `${ACCOUNT_PREFIX}[${account.name}](${data})`;
    }
    visitAccountData(accountData) {
        const struct = (0, Visitor_1.visit)(accountData.struct, this);
        const link = accountData.link ? `;${(0, Visitor_1.visit)(accountData.link, this)}` : '';
        return struct + link;
    }
    visitInstruction(instruction) {
        const accounts = instruction.accounts.map((account) => (0, Visitor_1.visit)(account, this));
        const dataArgs = (0, Visitor_1.visit)(instruction.dataArgs, this);
        const extraArgs = (0, Visitor_1.visit)(instruction.extraArgs, this);
        return (`${INSTRUCTION_PREFIX}[${instruction.name}](` +
            `accounts(${accounts.join(',')}),${dataArgs},${extraArgs}` +
            `)`);
    }
    visitInstructionAccount(instructionAccount) {
        return instructionAccount.name;
    }
    visitInstructionDataArgs(instructionDataArgs) {
        const struct = (0, Visitor_1.visit)(instructionDataArgs.struct, this);
        const link = instructionDataArgs.link
            ? `;${(0, Visitor_1.visit)(instructionDataArgs.link, this)}`
            : '';
        return `dataArgs(${struct + link})`;
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        const struct = (0, Visitor_1.visit)(instructionExtraArgs.struct, this);
        const link = instructionExtraArgs.link
            ? `;${(0, Visitor_1.visit)(instructionExtraArgs.link, this)}`
            : '';
        return `extraArgs(${struct + link})`;
    }
    visitDefinedType(definedType) {
        const data = (0, Visitor_1.visit)(definedType.data, this);
        return `${TYPE_PREFIX}[${definedType.name}](${data})`;
    }
    visitError(error) {
        return `${ERROR_PREFIX}[${error.name}]`;
    }
    visitArrayType(arrayType) {
        const child = (0, Visitor_1.visit)(arrayType.child, this);
        const size = (0, shared_1.displaySizeStrategy)(arrayType.size);
        return `array(${child};${size})`;
    }
    visitLinkType(linkType) {
        return `link(${linkType.name};${linkType.importFrom})`;
    }
    visitEnumType(enumType) {
        const children = enumType.variants.map((variant) => (0, Visitor_1.visit)(variant, this));
        return `enum(${children.join(',')})`;
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        return enumEmptyVariantType.name;
    }
    visitEnumStructVariantType(enumStructVariantType) {
        const child = (0, Visitor_1.visit)(enumStructVariantType.struct, this);
        return `${enumStructVariantType.name}:${child}`;
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        const child = (0, Visitor_1.visit)(enumTupleVariantType.tuple, this);
        return `${enumTupleVariantType.name}:${child}`;
    }
    visitMapType(mapType) {
        const key = (0, Visitor_1.visit)(mapType.key, this);
        const value = (0, Visitor_1.visit)(mapType.value, this);
        const size = (0, shared_1.displaySizeStrategy)(mapType.size);
        return `map(${key},${value};${size})`;
    }
    visitOptionType(optionType) {
        const child = (0, Visitor_1.visit)(optionType.child, this);
        const prefix = (0, Visitor_1.visit)(optionType.prefix, this);
        const fixed = optionType.fixed ? ';fixed' : '';
        return `option(${child};${prefix + fixed})`;
    }
    visitSetType(setType) {
        const child = (0, Visitor_1.visit)(setType.child, this);
        const size = (0, shared_1.displaySizeStrategy)(setType.size);
        return `set(${child};${size})`;
    }
    visitStructType(structType) {
        const children = structType.fields.map((field) => (0, Visitor_1.visit)(field, this));
        return `struct(${children.join(',')})`;
    }
    visitStructFieldType(structFieldType) {
        const child = (0, Visitor_1.visit)(structFieldType.child, this);
        return `${structFieldType.name}:${child}`;
    }
    visitTupleType(tupleType) {
        const children = tupleType.children.map((child) => (0, Visitor_1.visit)(child, this));
        return `tuple(${children.join(',')})`;
    }
    visitBoolType(boolType) {
        return boolType.toString();
    }
    visitBytesType(bytesType) {
        return bytesType.toString();
    }
    visitNumberType(numberType) {
        return numberType.toString();
    }
    visitNumberWrapperType(numberWrapperType) {
        const item = (0, Visitor_1.visit)(numberWrapperType.number, this);
        const { wrapper } = numberWrapperType;
        switch (wrapper.kind) {
            case 'DateTime':
                return `DateTime(${item})`;
            case 'Amount':
                return `Amount(${item},${wrapper.identifier},${wrapper.decimals})`;
            case 'SolAmount':
                return `SolAmount(${item})`;
            default:
                return item;
        }
    }
    visitPublicKeyType() {
        return 'publicKey';
    }
    visitStringType(stringType) {
        return stringType.toString();
    }
    displayArrayLikeSize(size) {
        if (size.kind === 'fixed')
            return `${size.value}`;
        if (size.kind === 'prefixed')
            return (0, Visitor_1.visit)(size.prefix, this);
        return 'remainder';
    }
}
exports.GetNodeInlineStringVisitor = GetNodeInlineStringVisitor;
//# sourceMappingURL=GetNodeInlineStringVisitor.js.map