"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateSubInstructionsFromEnumArgsVisitor = void 0;
const shared_1 = require("../../shared");
const logs_1 = require("../../shared/logs");
const nodes = __importStar(require("../../nodes"));
const TransformNodesVisitor_1 = require("./TransformNodesVisitor");
const FlattenStructVisitor_1 = require("./FlattenStructVisitor");
class CreateSubInstructionsFromEnumArgsVisitor extends TransformNodesVisitor_1.TransformNodesVisitor {
    constructor(map) {
        const transforms = Object.entries(map).map(([selector, argNameInput]) => {
            const selectorStack = selector.split('.');
            const name = selectorStack.pop();
            return {
                selector: { kind: 'instructionNode', stack: selectorStack, name },
                transformer: (node) => {
                    var _a, _b;
                    nodes.assertInstructionNode(node);
                    const argFields = node.dataArgs.struct.fields;
                    const argName = (0, shared_1.mainCase)(argNameInput);
                    const argFieldIndex = argFields.findIndex((field) => field.name === argName);
                    const argField = argFieldIndex >= 0 ? argFields[argFieldIndex] : null;
                    if (!argField) {
                        (0, logs_1.logWarn)(`Could not find instruction argument [${argName}].`);
                        return node;
                    }
                    let argType;
                    if (nodes.isEnumTypeNode(argField.child)) {
                        argType = argField.child;
                    }
                    else if (nodes.isLinkTypeNode(argField.child) &&
                        this.allDefinedTypes.has(argField.child.name)) {
                        const linkedType = (_b = (_a = this.allDefinedTypes.get(argField.child.name)) === null || _a === void 0 ? void 0 : _a.data) !== null && _b !== void 0 ? _b : null;
                        nodes.assertEnumTypeNode(linkedType);
                        argType = linkedType;
                    }
                    else {
                        (0, logs_1.logWarn)(`Could not find an enum type for ` +
                            `instruction argument [${argName}].`);
                        return node;
                    }
                    const subInstructions = argType.variants.map((variant, index) => {
                        const subName = (0, shared_1.mainCase)(`${node.name} ${variant.name}`);
                        const subFields = argFields.slice(0, argFieldIndex);
                        subFields.push(nodes.structFieldTypeNode({
                            name: `${subName}Discriminator`,
                            child: nodes.numberTypeNode('u8'),
                            defaultsTo: {
                                strategy: 'omitted',
                                value: nodes.vScalar(index),
                            },
                        }));
                        if (nodes.isEnumStructVariantTypeNode(variant)) {
                            subFields.push(nodes.structFieldTypeNode({
                                ...argField,
                                child: variant.struct,
                            }));
                        }
                        else if (nodes.isEnumTupleVariantTypeNode(variant)) {
                            subFields.push(nodes.structFieldTypeNode({
                                ...argField,
                                child: variant.tuple,
                            }));
                        }
                        subFields.push(...argFields.slice(argFieldIndex + 1));
                        return nodes.instructionNode({
                            ...node,
                            name: subName,
                            dataArgs: nodes.instructionDataArgsNode({
                                ...node.dataArgs,
                                name: `${subName}InstructionData`,
                                struct: (0, FlattenStructVisitor_1.flattenStruct)(nodes.structTypeNode(subFields)),
                            }),
                            extraArgs: nodes.instructionExtraArgsNode({
                                ...node.extraArgs,
                                name: `${subName}InstructionExtra`,
                            }),
                        });
                    });
                    return nodes.instructionNode({
                        ...node,
                        subInstructions: [...node.subInstructions, ...subInstructions],
                    });
                },
            };
        });
        super(transforms);
        this.map = map;
        this.allDefinedTypes = new Map();
    }
    visitRoot(root) {
        nodes.getAllDefinedTypes(root).forEach((type) => {
            this.allDefinedTypes.set(type.name, type);
        });
        return super.visitRoot(root);
    }
}
exports.CreateSubInstructionsFromEnumArgsVisitor = CreateSubInstructionsFromEnumArgsVisitor;
//# sourceMappingURL=CreateSubInstructionsFromEnumArgsVisitor.js.map