"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformNodesVisitor = void 0;
const BaseNodeOrNullVisitor_1 = require("../BaseNodeOrNullVisitor");
const NodeSelector_1 = require("../NodeSelector");
const NodeStack_1 = require("../NodeStack");
class TransformNodesVisitor extends BaseNodeOrNullVisitor_1.BaseNodeOrNullVisitor {
    constructor(transforms) {
        super();
        this.stack = new NodeStack_1.NodeStack();
        this.program = null;
        this.transforms = transforms.map((transform) => ({
            ...transform,
            selector: (0, NodeSelector_1.toNodeSelectorFunction)(transform.selector),
        }));
    }
    visitRoot(root) {
        this.stack.push(root);
        const visitedRoot = super.visitRoot(root);
        this.stack.pop();
        return this.applyTransforms(visitedRoot);
    }
    visitProgram(program) {
        this.stack.push(program);
        this.program = program;
        const visitedProgram = super.visitProgram(program);
        this.program = null;
        this.stack.pop();
        return this.applyTransforms(visitedProgram);
    }
    visitAccount(account) {
        this.stack.push(account);
        const visitedAccount = super.visitAccount(account);
        this.stack.pop();
        return this.applyTransforms(visitedAccount);
    }
    visitAccountData(accountData) {
        this.stack.push(accountData);
        const visitedAccountData = super.visitAccountData(accountData);
        this.stack.pop();
        return this.applyTransforms(visitedAccountData);
    }
    visitInstruction(instruction) {
        this.stack.push(instruction);
        const visitedInstruction = super.visitInstruction(instruction);
        this.stack.pop();
        return this.applyTransforms(visitedInstruction);
    }
    visitInstructionAccount(instructionAccount) {
        this.stack.push(instructionAccount);
        const visitedInstructionAccount = super.visitInstructionAccount(instructionAccount);
        this.stack.pop();
        return this.applyTransforms(visitedInstructionAccount);
    }
    visitInstructionDataArgs(instructionDataArgs) {
        this.stack.push(instructionDataArgs);
        const visitedInstructionDataArgs = super.visitInstructionDataArgs(instructionDataArgs);
        this.stack.pop();
        return this.applyTransforms(visitedInstructionDataArgs);
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        this.stack.push(instructionExtraArgs);
        const visitedInstructionExtraArgs = super.visitInstructionExtraArgs(instructionExtraArgs);
        this.stack.pop();
        return this.applyTransforms(visitedInstructionExtraArgs);
    }
    visitDefinedType(definedType) {
        this.stack.push(definedType);
        const visitedDefinedType = super.visitDefinedType(definedType);
        this.stack.pop();
        return this.applyTransforms(visitedDefinedType);
    }
    visitError(error) {
        this.stack.push(error);
        const visitedError = super.visitError(error);
        this.stack.pop();
        return this.applyTransforms(visitedError);
    }
    visitArrayType(arrayType) {
        this.stack.push(arrayType);
        const visitedTypeArray = super.visitArrayType(arrayType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeArray);
    }
    visitLinkType(linkType) {
        this.stack.push(linkType);
        const visitedTypeDefinedLink = super.visitLinkType(linkType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeDefinedLink);
    }
    visitEnumType(enumType) {
        this.stack.push(enumType);
        const visitedTypeEnum = super.visitEnumType(enumType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnum);
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        this.stack.push(enumEmptyVariantType);
        const visitedTypeEnumEmptyVariant = super.visitEnumEmptyVariantType(enumEmptyVariantType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumEmptyVariant);
    }
    visitEnumStructVariantType(enumStructVariantType) {
        this.stack.push(enumStructVariantType);
        const visitedTypeEnumStructVariant = super.visitEnumStructVariantType(enumStructVariantType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumStructVariant);
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        this.stack.push(enumTupleVariantType);
        const visitedTypeEnumTupleVariant = super.visitEnumTupleVariantType(enumTupleVariantType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeEnumTupleVariant);
    }
    visitMapType(mapType) {
        this.stack.push(mapType);
        const visitedTypeMap = super.visitMapType(mapType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeMap);
    }
    visitOptionType(optionType) {
        this.stack.push(optionType);
        const visitedTypeOption = super.visitOptionType(optionType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeOption);
    }
    visitSetType(setType) {
        this.stack.push(setType);
        const visitedTypeSet = super.visitSetType(setType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeSet);
    }
    visitStructType(structType) {
        this.stack.push(structType);
        const visitedTypeStruct = super.visitStructType(structType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeStruct);
    }
    visitStructFieldType(structFieldType) {
        this.stack.push(structFieldType);
        const visitedTypeStructField = super.visitStructFieldType(structFieldType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeStructField);
    }
    visitTupleType(tupleType) {
        this.stack.push(tupleType);
        const visitedTypeTuple = super.visitTupleType(tupleType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeTuple);
    }
    visitBoolType(boolType) {
        this.stack.push(boolType);
        const visitedTypeBool = super.visitBoolType(boolType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeBool);
    }
    visitBytesType(bytesType) {
        this.stack.push(bytesType);
        const visitedTypeBytes = super.visitBytesType(bytesType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeBytes);
    }
    visitNumberType(numberType) {
        this.stack.push(numberType);
        const visitedTypeNumber = super.visitNumberType(numberType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeNumber);
    }
    visitNumberWrapperType(numberWrapperType) {
        this.stack.push(numberWrapperType);
        const visitedTypeNumberWrapper = super.visitNumberWrapperType(numberWrapperType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeNumberWrapper);
    }
    visitPublicKeyType(publicKeyType) {
        this.stack.push(publicKeyType);
        const visitedTypePublicKey = super.visitPublicKeyType(publicKeyType);
        this.stack.pop();
        return this.applyTransforms(visitedTypePublicKey);
    }
    visitStringType(stringType) {
        this.stack.push(stringType);
        const visitedTypeString = super.visitStringType(stringType);
        this.stack.pop();
        return this.applyTransforms(visitedTypeString);
    }
    applyTransforms(node) {
        if (node === null)
            return null;
        const stack = this.stack.clone();
        const { program } = this;
        return this.transforms
            .filter(({ selector }) => selector(node, stack, program))
            .reduce((acc, { transformer }) => acc === null ? null : transformer(acc, stack, program), node);
    }
}
exports.TransformNodesVisitor = TransformNodesVisitor;
//# sourceMappingURL=TransformNodesVisitor.js.map